package com.hosh.verse.common.quadtree;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Map;

import com.badlogic.gdx.math.Vector2;

import dacite.DBAnnotation;

public abstract class AbstractQuadNode<T> {

	protected final static int MAX_ELEMENTS = 4;
	protected final static int MAX_DEPTH = 4;
	private final Vector2 size;
	private final Vector2 startCoords;
	private final int maxDepth;
	private final int maxElements;
	private final int depth;

	public static enum Quadrant {
		TOP_LEFT, BOTTOM_RIGHT, BOTTOM_LEFT, TOP_RIGHT
	}

	public AbstractQuadNode(final Vector2 startCoordinates, final Vector2 bounds, final int depth) {
		this(startCoordinates, bounds, depth, MAX_ELEMENTS, MAX_DEPTH);
	}

	public AbstractQuadNode(final Vector2 startCoordinates, final Vector2 size, final int depth, final int maxDepth, final int maxElements) {
		this.startCoords = startCoordinates;
		this.size = size;
		this.maxDepth = maxDepth;
		this.maxElements = maxElements;
		this.depth = depth;
	}

	public Vector2 getSize() {
		return size;
	}

	public int getMaxDepth() {
		return this.maxDepth;
	}

	public int getDepth() {
		return this.depth;
	}

	public Vector2 getStartCoordinates() {
		return startCoords;
	}

	public int getMaxElements() {
		return maxElements;
	}

	public abstract void subdivide();

	public abstract Map<Quadrant, ? extends AbstractQuadNode<T>> getChildren();

	public abstract void clear();

	/*public void witch1() {
		try {
			final Connection connect = DriverManager.getConnection("jdbc:mysql://localhost:3306/verse_db", "root", "2543120");
			final Statement stmt = connect.createStatement();
			final String query = "select area, gender from npc_location;";
			final ResultSet rs = stmt.executeQuery(query);

			final int area = rs.getInt("area");
			DBAnnotation.annoate("area", "npc_location", "area", true);
			final int gender = rs.getInt("gender");
			DBAnnotation.annoate("gender", "npc_location", "gender", true);

			int witch;
			if (area == 5) {
				if (gender == 0) {
					witch = 1;
					final String sql = "update npc_location set  witch = " + witch + "where area = " + area + " and gender = " + gender
							+ ";";
					stmt.executeUpdate(sql);
					DBAnnotation.annoate("witch", "npc_location", "witch", false);
				}
			}

		} catch (final SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void witch2() {
		try {
			final Connection connect = DriverManager.getConnection("jdbc:mysql://localhost:3306/verse_db", "root", "2543120");
			final Statement stmt = connect.createStatement();
			final String query = "select area, gender from npc_location;";
			final ResultSet rs = stmt.executeQuery(query);

			final int area = rs.getInt("area");
			DBAnnotation.annoate("area", "npc_location", "area", true);
			final int gender = rs.getInt("gender");
			DBAnnotation.annoate("gender", "npc_location", "gender", true);

			int witch;
			if (area == 5) {
				if (gender == 1) {
					witch = 1;
					final String sql = "update npc_location set  witch = " + witch + "where area = " + area + " and gender = " + gender
							+ ";";
					stmt.executeUpdate(sql);
					DBAnnotation.annoate("witch", "npc_location", "witch", false);
				}
			}

		} catch (final SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}*/
	public void underage_player1() {
		try {
		    final Connection connect = DriverManager.getConnection(
			    "jdbc:mysql://localhost:3306/verse_db", "root", "2543120");
		    final Statement stmt = connect.createStatement();
		    final String query = "select age from player;";
		    final ResultSet rs = stmt.executeQuery(query);

		    final int age = rs.getInt("age");
		    DBAnnotation.annoate("age", "player", "age", true);
		  
		    int underage_player;
		    if (age <= 25) {
			underage_player = 1;
			final String sql = "update player set  underage_player = "
				+ underage_player + "where age = " + age + ";";
			stmt.executeUpdate(sql);
			DBAnnotation.annoate("underage_player", "player",
				"underage_player", false);
		    }

		} catch (final SQLException e) { // TODO Auto-generated catch block
		    e.printStackTrace();
		}
	    }
	    
	    public void underage_player2() {
		try {
		    final Connection connect = DriverManager.getConnection(
			    "jdbc:mysql://localhost:3306/verse_db", "root", "2543120");
		    final Statement stmt = connect.createStatement();
		    final String query = "select age from player;";
		    final ResultSet rs = stmt.executeQuery(query);

		    final int age = rs.getInt("age");
		    DBAnnotation.annoate("age", "player", "age", true);
		  
		    int underage_player;
		    if (age > 50) {
			underage_player = 1;
			final String sql = "update player set  underage_player = "
				+ underage_player + "where age = " + age + ";";
			stmt.executeUpdate(sql);
			DBAnnotation.annoate("underage_player", "player",
				"underage_player", false);
		    }

		} catch (final SQLException e) { // TODO Auto-generated catch block
		    e.printStackTrace();
		}
	    }
}
